@@ -170,6 +170,7 @@ DEFINE_TEST(test_pax_filename_encoding_CP932)
 DEFINE_TEST(test_pax_filename_encoding_KOI8R_BINARY)
 DEFINE_TEST(test_pax_filename_encoding_KOI8R_CP1251)
 DEFINE_TEST(test_pax_filename_encoding)
+DEFINE_TEST(test_pax_xattr_header)
 DEFINE_TEST(test_read_data_large)
 DEFINE_TEST(test_read_disk)
 DEFINE_TEST(test_read_disk_directory_traversals)
@@ -200,6 +201,7 @@ DEFINE_TEST(test_read_format_7zip_encryption_data)
 DEFINE_TEST(test_read_format_7zip_encryption_header)
 DEFINE_TEST(test_read_format_7zip_encryption_partially)
 DEFINE_TEST(test_read_format_7zip_malformed)
+DEFINE_TEST(test_read_format_7zip_packinfo_digests)
 DEFINE_TEST(test_read_format_ar)
 DEFINE_TEST(test_read_format_cab)
 DEFINE_TEST(test_read_format_cab_filename)
@@ -263,6 +265,7 @@ DEFINE_TEST(test_read_format_isozisofs_bz2)
 DEFINE_TEST(test_read_format_lha_bugfix_0)
 DEFINE_TEST(test_read_format_lha)
 DEFINE_TEST(test_read_format_lha_filename)
+DEFINE_TEST(test_read_format_lha_filename_UTF16)
 DEFINE_TEST(test_read_format_mtree)
 DEFINE_TEST(test_read_format_mtree_filenames_only)
 DEFINE_TEST(test_read_format_mtree_nochange)
@@ -311,6 +314,10 @@ DEFINE_TEST(test_read_format_rar5_invalid_dict_reference)
 DEFINE_TEST(test_read_format_rar5_distance_overflow)
 DEFINE_TEST(test_read_format_rar5_nonempty_dir_stream)
 DEFINE_TEST(test_read_format_rar5_fileattr)
+DEFINE_TEST(test_read_format_rar5_different_window_size)
+DEFINE_TEST(test_read_format_rar5_arm_filter_on_window_boundary)
+DEFINE_TEST(test_read_format_rar5_different_solid_window_size)
+DEFINE_TEST(test_read_format_rar5_different_winsize_on_merge)
 DEFINE_TEST(test_read_format_rar_set_format)
 DEFINE_TEST(test_read_format_rar_basic)
 DEFINE_TEST(test_read_format_rar_subblock)
@@ -332,6 +339,7 @@ DEFINE_TEST(test_read_format_rar_multivolume_seek_data)
 DEFINE_TEST(test_read_format_rar_multivolume_seek_multiple_files)
 DEFINE_TEST(test_read_format_rar_multivolume_uncompressed_files)
 DEFINE_TEST(test_read_format_rar_ppmd_use_after_free)
+DEFINE_TEST(test_read_format_rar_ppmd_use_after_free2)
 DEFINE_TEST(test_read_format_rar_encryption_data)
 DEFINE_TEST(test_read_format_rar_encryption_header)
 DEFINE_TEST(test_read_format_rar_encryption_partially)
