@@ -45,7 +45,7 @@ DEFINE_TEST(test_read_extract)
 	/* Create a new archive in memory containing various types of entries. */
 	assert((a = archive_write_new()) != NULL);
 	assertA(0 == archive_write_set_format_ustar(a));
-	assertA(0 == archive_write_set_compression_none(a));
+	assertA(0 == archive_write_add_filter_none(a));
 	assertA(0 == archive_write_open_memory(a, buff, BUFF_SIZE, &used));
 	/* A directory to be restored with EXTRACT_PERM. */
 	++numEntries;
@@ -161,7 +161,7 @@ DEFINE_TEST(test_read_extract)
 	assertIsDir("dir4/b", 0755);
 	assertIsDir("dir4/c", 0711);
 	if (canSymlink())
-		assertIsSymlink("symlink", "file");
+		assertIsSymlink("symlink", "file", 0);
 
 	free(buff);
 	free(file_buff);
